package com.displaynone.acss.components.acs;

import com.displaynone.acss.components.acs.models.gate.GateModel;
import com.displaynone.acss.components.acs.models.gate.service.GateService;
import com.displaynone.acss.components.acs.models.visit.VisitModel;
import com.displaynone.acss.components.acs.models.visit.service.VisitService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Optional;

@Slf4j
@Component
@RequiredArgsConstructor
public class ACSComponent {
    private final GateService gateService;
    private final VisitService visitService;

    public Optional<GateModel> getGateById(Long gateId) {
        return gateService.getGateById(gateId);
    }

    public GateModel getGateByIdStrict(Long gateId) {
        return gateService.getGateByIdStrict(gateId);
    }

    public Optional<GateModel> getGateByCode(Long code) {
        return gateService.getGateByCode(code);
    }

    public GateModel getGateByCodeStrict(Long code) {
        return gateService.getGateByCodeStrict(code);
    }

    public boolean isGateExists(Long gateId) {
        return gateService.isGateExists(gateId);
    }

    public boolean isGateCodeValid(Long code) {
        return gateService.isGateCodeValid(code);
    }

    public List<VisitModel> getVisitsByGateId(Long gateId) {
        return visitService.getByGateId(gateId);
    }

    public List<VisitModel> getVisitsByUserId(Long userId) {
        return visitService.getByUserId(userId);
    }

    public Page<VisitModel> getVisitsByGateIdPaginated(Long gateId, Pageable pageable) {
        return visitService.getVisitsByGateIdPaginated(gateId, pageable);
    }

    public Page<VisitModel> getVisitsByUserIdPaginated(Long userId, Pageable pageable) {
        return visitService.getVisitsByUserIdPaginated(userId, pageable);
    }

    public void updateGate(Long gateId, GateModel gate) {
        gateService.updateGate(gateId, gate);
    }

    public void createVisit(VisitModel visit) {
        visitService.createVisit(visit);
    }

    public void createVisit(Long userId, Long gateId) {
        VisitModel visit = new VisitModel();
        visit.setUserId(userId);
        visit.setGateId(gateId);
        createVisit(visit);
    }
}
