package com.displaynone.acss.components.acs.models.gate.service;

import com.displaynone.acss.components.acs.models.gate.GateModel;
import com.displaynone.acss.components.acs.models.gate.GateRepository;
import com.displaynone.acss.exception.generics.NotFoundHTTPException;
import com.displaynone.acss.utils.GlobalUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Optional;


@Component
@RequiredArgsConstructor
public class GateServiceImpl implements GateService {
    private final GateRepository gateRepository;

    private void saveGate(GateModel gate) {
        gateRepository.save(gate);
    }

    @Override
    public Optional<GateModel> getGateById(Long gateId) {
        return gateRepository.findById(gateId);
    }

    @Override
    public GateModel getGateByIdStrict(Long gateId) {
        Optional<GateModel> model = gateRepository.findById(gateId);
        if (model.isEmpty()) throw new NotFoundHTTPException("Gate not found");
        return model.get();
    }

    @Override
    public Optional<GateModel> getGateByCode(Long code) {
        return gateRepository.findByCode(code);
    }

    @Override
    public GateModel getGateByCodeStrict(Long code) {
        Optional<GateModel> model = getGateByCode(code);
        if (model.isEmpty()) throw new NotFoundHTTPException("Gate not found");
        return model.get();
    }

    @Override
    public boolean isGateExists(Long gateId) {
        return getGateById(gateId).isPresent();
    }

    @Override
    public boolean isGateCodeValid(Long code) {
        return getGateByCode(code).isPresent();
    }

    public void updateGate(Long gateId, GateModel gateUpdate) {
        GateModel existingGate = getGateByIdStrict(gateId);

        GlobalUtils.updateIfNotNull(gateUpdate.getCode(), existingGate::setCode);
        GlobalUtils.updateIfNotNull(gateUpdate.isActive(), existingGate::setActive);
        GlobalUtils.updateIfNotNull(gateUpdate.isACSBlocked(), existingGate::setACSBlocked);

        saveGate(existingGate);
    }
}
