package com.displaynone.acss.components.auth.models.user.service;

import com.displaynone.acss.components.auth.models.user.UserModel;
import com.displaynone.acss.components.auth.models.user.UserRepository;
import com.displaynone.acss.exception.generics.NotFoundHTTPException;
import com.displaynone.acss.utils.GlobalUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.Optional;

@Service
@RequiredArgsConstructor
public class UserServiceImpl implements UserService {
    private final UserRepository userRepository;

    private void saveUser(UserModel userUpdated) {
        userRepository.save(userUpdated);
    }

    @Override
    public Optional<UserModel> findByLogin(String login) {
        return userRepository.findByLogin(login);
    }

    @Override
    public UserModel findByLoginStrict(String login) {
        Optional<UserModel> user = findByLogin(login);
        if (user.isEmpty()) throw new NotFoundHTTPException("User not found");
        return user.get();
    }

    @Override
    public void updateUserByLogin(String login, UserModel userUpdated) {
        UserModel existingUser = findByLoginStrict(login);

        GlobalUtils.updateIfNotNull(userUpdated.getName(), existingUser::setName);
        GlobalUtils.updateIfNotNull(userUpdated.getPhoto(), existingUser::setPhoto);
        GlobalUtils.updateIfNotNull(userUpdated.getPosition(), existingUser::setPosition);
        GlobalUtils.updateIfNotNull(userUpdated.getPassword(), existingUser::setPassword);
        GlobalUtils.updateIfNotNull(userUpdated.isActive(), existingUser::setActive);
        GlobalUtils.updateIfNotNull(userUpdated.isACSBlocked(), existingUser::setACSBlocked);

        saveUser(existingUser);
    }
}
