package com.displaynone.acss.controllers.auth;

import com.displaynone.acss.components.auth.AuthComponent;
import com.displaynone.acss.components.auth.models.AuthTokenPair;
import com.displaynone.acss.components.auth.models.user.UserModel;
import com.displaynone.acss.exception.generics.BadRequestHTTPException;
import com.displaynone.acss.exception.generics.NotFoundHTTPException;
import com.displaynone.acss.exception.generics.UnauthorizedHTTPException;
import com.displaynone.acss.utils.Pair;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/api/auth")
@RequiredArgsConstructor
public class AuthController {
    private final AuthComponent authComponent;

    @PostMapping("/login")
    public ResponseEntity<AuthTokenPair> login(@RequestBody LoginRQB body) {
        String login = body.getLogin();
        String password = body.getPassword();

        if (login == null | password == null) throw new BadRequestHTTPException("Auth credentials not specified");

        Pair<UserModel, AuthTokenPair> authInfo;

        try {
            authInfo = authComponent.authenticate(login, password);
        } catch (NotFoundHTTPException e) {
            throw new UnauthorizedHTTPException(e.getMessage());
        }

        return ResponseEntity.ok(authInfo.getSecond());
    }

    @PostMapping("/refresh")
    public ResponseEntity<?> refreshToken(@RequestBody TokenRefreshRQB body) {
        String requestRefreshToken = body.getRefreshToken();
        if (requestRefreshToken == null) throw new BadRequestHTTPException("Refresh token not specified");

        AuthTokenPair authTokenPair;
        try {
            authTokenPair = authComponent.refresh(requestRefreshToken);
        } catch (NotFoundHTTPException e) {
            throw new UnauthorizedHTTPException(e.getMessage());
        }

        return ResponseEntity.ok(authTokenPair);
    }
}