package com.displaynone.acss.controllers.user;

import com.displaynone.acss.components.auth.AuthComponent;
import com.displaynone.acss.components.auth.models.user.UserDTO;
import com.displaynone.acss.components.auth.models.user.UserMapper;
import com.displaynone.acss.components.auth.models.user.UserModel;
import com.displaynone.acss.utils.GlobalUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/api/users")
@RequiredArgsConstructor
public class UserController {
    private final AuthComponent authComponent;

    @GetMapping("/me")
    public ResponseEntity<UserDTO> getMe() {
        UserModel user = (UserModel) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        return ResponseEntity.ok(UserMapper.convertToDTO(user));
    }

    @GetMapping("/login/{login}")
    public ResponseEntity<UserDTO> getUserByLogin(@PathVariable String login) {
        UserModel user = authComponent.getUserByLoginStrict(login);
        return ResponseEntity.ok(UserMapper.convertToDTO(user));
    }

    @PatchMapping("/login/{login}")
    public ResponseEntity<Void> updateUserByLogin(@PathVariable String login, @RequestBody UserUpdateRQB body) {
        UserModel userUpdated = new UserModel();

        GlobalUtils.updateIfNotNull(body.getName(), userUpdated::setName);
        GlobalUtils.updateIfNotNull(body.getPhoto(), userUpdated::setPhoto);
        GlobalUtils.updateIfNotNull(body.getPosition(), userUpdated::setPosition);
        GlobalUtils.updateIfNotNull(body.getPassword(), userUpdated::setPassword);
        GlobalUtils.updateIfNotNull(body.getIsActive(), userUpdated::setActive);
        GlobalUtils.updateIfNotNull(body.getIsACSBlocked(), userUpdated::setACSBlocked);

        authComponent.updateUserByLogin(login, userUpdated);
        return ResponseEntity.ok(null);
    }
}
