package com.displaynone.acss.exception.handler;

import com.displaynone.acss.exception.*;
import com.displaynone.acss.exception.generics.*;
import com.fasterxml.jackson.databind.exc.InvalidFormatException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.lang.Nullable;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.resource.NoResourceFoundException;

@RestControllerAdvice
public class GlobalExceptionHandler {
    public static ResponseEntity<ErrorResponsePayload> buildErrorResponse(Exception e, HttpStatus status, @Nullable String message) {
        ErrorResponsePayload errorResponsePayload = new ErrorResponsePayload(
                status.value(),
                status.getReasonPhrase(),
                message == null ? e.getMessage() : message
        );
        return new ResponseEntity<>(errorResponsePayload, status);
    }

    @ExceptionHandler(Exception.class)
    public ResponseEntity<ErrorResponsePayload> handleException(Exception e, WebRequest request) {
        Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);
        logger.error("Unexpected error occurred", e);
        return buildErrorResponse(e, HttpStatus.INTERNAL_SERVER_ERROR, "Unexpected error occurred");
    }

    @ExceptionHandler(BadRequestHTTPException.class)
    public ResponseEntity<ErrorResponsePayload> handleBadBadRequestHTTPException(BadRequestHTTPException e, WebRequest request) {
        return buildErrorResponse(e, HttpStatus.BAD_REQUEST, null);
    }

    @ExceptionHandler(NotFoundHTTPException.class)
    public ResponseEntity<ErrorResponsePayload> handleNotFoundHTTPException(NotFoundHTTPException e, WebRequest request) {
        return buildErrorResponse(e, HttpStatus.NOT_FOUND, null);
    }

    @ExceptionHandler(AlreadyExistsHTTPException.class)
    public ResponseEntity<ErrorResponsePayload> handleAlreadyExistsHTTPException(AlreadyExistsHTTPException e, WebRequest request) {
        return buildErrorResponse(e, HttpStatus.CONFLICT, null);
    }

    @ExceptionHandler(HttpMessageNotReadableException.class)
    public ResponseEntity<ErrorResponsePayload> handleHttpMessageNotReadableException(InvalidFormatException e, WebRequest request) {
        return buildErrorResponse(e, HttpStatus.BAD_REQUEST, "Bad request payload format: failed to parse");
    }

    @ExceptionHandler(HttpRequestMethodNotSupportedException.class)
    public ResponseEntity<ErrorResponsePayload> handleHttpRequestMethodNotSupportedException(HttpRequestMethodNotSupportedException e, WebRequest request) {
        return buildErrorResponse(e, HttpStatus.METHOD_NOT_ALLOWED, null);
    }

    @ExceptionHandler(UnauthorizedHTTPException.class)
    public ResponseEntity<ErrorResponsePayload> handleUnauthorizedHTTPException(UnauthorizedHTTPException e, WebRequest request) {
        return buildErrorResponse(e, HttpStatus.UNAUTHORIZED, null);
    }

    @ExceptionHandler(InternalAuthenticationServiceException.class)
    public ResponseEntity<ErrorResponsePayload> handleInternalAuthenticationServiceException(InternalAuthenticationServiceException e, WebRequest request) {
        Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);
        logger.error("Unexpected error occurred", e);

        return buildErrorResponse(e, HttpStatus.UNAUTHORIZED, "Unauthorized: unexpected error occurred");
    }

    @ExceptionHandler(NoResourceFoundException.class)
    public ResponseEntity<ErrorResponsePayload> handleNoResourceFoundException(NoResourceFoundException e, WebRequest request) {
        return buildErrorResponse(e, HttpStatus.NOT_FOUND, null);
    }

    @ExceptionHandler(ForbiddenHTTPException.class)
    public ResponseEntity<ErrorResponsePayload> handleForbiddenHTTPException(ForbiddenHTTPException e, WebRequest request) {
        return buildErrorResponse(e, HttpStatus.FORBIDDEN, null);
    }
}
